# Copyright (C) 2011, Aleksey Lim
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import sys
import logging
from os.path import join, exists, dirname, isdir
from gettext import gettext as _

from sugar_mothership import env, util, leases
from sugar_mothership.util import enforce


def import_dir(args):
    enforce(args, _('Command "import" requires PATH argument'))
    path = args.pop(0)
    enforce(isdir(path), _('Path %s is not a dir'), path)

    for filename in os.listdir(path):
        filepath = join(path, filename)
        if filename in ['lease.sig', 'leases.dat']:
            import_leases([filepath])
        elif filename == 'd-lease.sig':
            import_lease_delegations([filepath])
        elif filename == 'd-oats.sig':
            import_oats_delegations([filepath])
        else:
            logging.info(_('Unrecognized file %s'), filepath)


def import_leases(args):
    _import(_('lease'), args, leases.leases_path,
            lambda content: leases.Lease(content).serial)


def import_lease_delegations(args):
    _import(_('lease delegation'), args, leases.lease_delegations_path,
            lambda content: leases.Delegation(content).serial)


def import_oats_delegations(args):
    _import(_('OATS delegation'), args, leases.oats_delegations_path,
            lambda content: leases.Delegation(content).serial)


def _import(title, path, get_path_cb, get_serial_cb):

    def write(line):
        try:
            line = line.strip() + '\n'
            serial = get_serial_cb(line)
            dst_path = join(env.root.value, get_path_cb(serial))

            logging.info(_('Import %s for serial %s'), title, serial)

            if not exists(dirname(dst_path)):
                os.makedirs(dirname(dst_path))
            f = file(dst_path, 'w')
            f.write(line)
            f.close()
        except Exception:
            logging.exception(_('Cannot import %s "%s" from %s'),
                    title, line, path)

    if path:
        path = path.pop(0)
        fin = file(path)
    else:
        path = 'stdin'
        fin = sys.stdin
    try:
        json_text = ''

        for line in fin:
            line = line.strip()
            if not line:
                continue

            if json_text is not None and (json_text or line.startswith('[')):
                json_text += line
            else:
                json_text = None
                try:
                    write(line)
                except Exception:
                    logging.exception(_('Cannot import %s "%s" from %s'),
                            title, line, path)

        if json_text:
            try:
                # It seems that there is no need in checking `_version` value
                # since `contnent` entries have version info anyway
                _version, content = util.json.loads(json_text)
                for i in content.values() \
                        if isinstance(content, dict) else content:
                    write(i)
            except Exception:
                logging.exception(_('Cannot import %s from %s'), title, path)
    finally:
        if fin is not sys.stdin:
            fin.close()
